/*
 * NumeroMaquina.java
 *
 * Creado el 29 de noviembre de 2005, 11:35
 */


package computacion.practica2;

/**
 * @author Antonio Pallars, Vctor Jimnez
 *
 * MTODOS NUMRICOS
 * LICENCIATURA DE MATEMTICAS
 * UNIVERSIDAD DE MURCIA
 */
public class NumeroMaquina{
    private long mantisa;
    private long exponente;
    
    /**
     * La variable precisin es esttica y se utiliza para fijar el nmero de
     * dgitos de la mantisa. Por defecto tiene el valor 6 y siempre ha de
     * ser menor que 16.
     */
    static int precision = 6;
    
    /**
     * Constructor bsico de NumeroMaquina
     */
    public NumeroMaquina() {
        mantisa = 0;
        exponente = 0;
    }
    
    /**
     * Constructor de un NumeroMaquina definido por una mantisa (entero largo)
     * y un exponente (entero largo).
     */
    public NumeroMaquina(long mant, long exp) {
        long digmantisa;
        String s;
        
        if (mant != 0) {
            while((mant % 10) == 0) {
                mant = mant / 10;
            }
        }
        s = Long.toString(Math.abs(mant));
        digmantisa = s.length();
        if (digmantisa <= precision) {
            mantisa = mant;
        } else {
            mantisa = Math.round(mant * Math.pow(10, - digmantisa + precision));
        }
        exponente = exp;
    }
    
    /**
     * Constructor de un NumeroMaquina clonando el NumeroMaquina N.
     */
    public NumeroMaquina(NumeroMaquina N) {
        mantisa = N.mantisa;
        exponente = N.exponente;
    }
    
    /**
     * Mtodo pasaANMaquina(a) para pasar a un NumeroMaquina
     * el nmero double a.     
     */
    public static NumeroMaquina pasaANMaquina(double a) {
        long exp = 0;
        long mant;
        String s;
        
        if (a == 0) {
            return (new NumeroMaquina());
        }        
        if (Math.abs(a) >= 1) {
            while (Math.abs(a) >= 1) {
                exp = exp + 1;
                a = a / 10.;
            }
        } else {
            while (Math.abs(a) < 0.1) {
                exp = exp - 1;
                a = a * 10.;
            }
        }
        mant = Math.round(a * Math.pow(10, precision));
        s = Long.toString(Math.abs(mant));
        if (s.length() == precision + 1) {
            exp = exp + 1;
        }
        return (new NumeroMaquina(mant, exp));
    }
    
    /**
     * Mtodo pasaADouble() para pasar a un nmero double el NumeroMaquina a.
     */
    public double pasaADouble() {
        String s = Long.toString(Math.abs(mantisa));
        long digMantisa = s.length();
        double num = mantisa * Math.pow(10, - digMantisa + exponente);
        
        return num;
    }
    
    /**
     * Mtodos que para operar un NumeroMaquina M con otro NumeroMaquina N.
     * Por ejemplo, si M y N son dos NumeroMaquina
     * M.suma(N) representa el nmero de maquina
     * que resulta al sumar M y N.
     */
    public NumeroMaquina suma(NumeroMaquina N) {
        double suma = pasaADouble() + N.pasaADouble();
        
        return NumeroMaquina.pasaANMaquina(suma);
    }
    
    public NumeroMaquina resta(NumeroMaquina N) {
        double resta = pasaADouble() - N.pasaADouble();
        
        return NumeroMaquina.pasaANMaquina(resta);
    }
        
    public NumeroMaquina producto(NumeroMaquina N) {
        double producto = pasaADouble() * N.pasaADouble();
        
        return NumeroMaquina.pasaANMaquina(producto);
    }
    
    public NumeroMaquina div(NumeroMaquina N) {
        double div = pasaADouble() / N.pasaADouble();
        
        return NumeroMaquina.pasaANMaquina(div);
    }
    
    public NumeroMaquina sqrt() {
        double raiz = Math.sqrt(pasaADouble());
        
        return NumeroMaquina.pasaANMaquina(raiz);
    }
    
    /**
     * Mtodo que implementa el mtodo toString para poder imprimir 
     * los NumeroMaquina con el formato (.mantisa Eexponente).
     * Por ejemplo, si a = NumeroMaquina(312, -3) entonces
     * System.out.println(a) imprimir (.312 E-3).
     */
    public String toString() {
        String result = "(";
                
        if (mantisa < 0) {
            result = result + "-." + (-mantisa);
        } else {
            result = result + "." + mantisa;
        }
        result = result + " E" + exponente + ")";
        return result;
    }
        
    public static double errorAbsoluto(double real, NumeroMaquina aprox) {
        return Math.abs(real - aprox.pasaADouble());
    }
    
    public static double errorRelativo(double real, NumeroMaquina aprox) {
        return errorAbsoluto(real, aprox) / Math.abs(real);
    }
    
}